/*
 *  Licensed Materials - Property of IBM / HCL Technologies
 *  Restricted Material of IBM / HCL Technologies
 *  IBM Rational Test RealTime / HCL OneTest
 *  (c) Copyright IBM Corp. 2020 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *       |\      _,,,---,,_
 *       /,`.-'`'    -.  ;-;;,_
 *      |,4-  ) )-,_..;\ (  `'-'
 *     '---''(_/--'  `-'\_)
 *
 ********************************************
 */
/*
 * this file contains methods used to create model
 * it also contains global variables, as model and sizes
 */
#include <stdio.h>
#include <stdlib.h>
#include <stddef.h>
#include "core.h"
#include "display.h"


unsigned int size_x = 10;
unsigned int size_y = 10;
unsigned short int ** model = NULL;

int offset_x[] = {0,1,0,-1};
int offset_y[] = {1,0,-1,0};

static void removeWall(int x, int y, int dir);
static int getSuroundingFreeCell(int x, int y, int res[4]);
static int findRandomFreeDir(int x, int y);
static int findSlotToOpen(int *ret_x,int *ret_y);

int myrandom(int x) {
	/* random value between 0 and x */
	return (rand() % (x));
}

void initialize() {
	if ((size_x > 0) && (size_y > 0) && (size_x < 100) && (size_y < 100)) {
		int i = 0;
		int j = 0;
		int current_x = 0;
		int current_y = 0;
		int dir = 0;

		/* fill matrix */
		model = (unsigned short int **)malloc((sizeof(unsigned short int))*size_x*size_y);
		for (i=0; i<size_x; i++) {
			model[i] = (unsigned short int *)malloc((sizeof(unsigned short int))*size_y);
			for (j=0; j<size_y; j++) {
				model[i][j] = WALL_ALL;
			}
		}
		for (i=0; i<size_x; i++) {
			model[i][0] = WALL_UP;
			model[i][size_y-1] = WALL_DOWN;
		}
		for (j=0; j<size_y; j++) {
			model[0][j] = WALL_RIGHT;
			model[size_x-1][j] = WALL_LEFT;
		}
		model[0][0] = 0;
		model[size_x-1][0] = 0;
		model[0][size_y-1] = 0;
		model[size_x-1][size_y-1] = 0;
		current_x = (int)(size_x/2);
		current_y = (int)(size_y/2);
		/* let's start to dig */
		do {
			do {
				dir = findRandomFreeDir(current_x,current_y);
				if (dir != -1) {
					removeWall(current_x,current_y,dir);
					current_x += offset_x[dir];
					current_y += offset_y[dir];
				}
			} while (dir != -1);
		} while (findSlotToOpen(&current_x,&current_y));
		removeWall(1+myrandom(size_x-3),size_y-2,UP);
	}
}

static int findSlotToOpen(int *ret_x,int *ret_y) {
	int i = 0;
	int j = 0;
	int k = 0;
	int xx;
	int yy;

	for (i = 1; i < (size_x -1); i++) {
		for (j = 1; j < (size_y - 1); j++) {
			/* find an openned cell */
			if (model[i][j] != WALL_ALL) {
				for (k = 0; k < 4; k++) {
					xx = i + offset_x[k];
					yy = j + offset_y[k];
					/* look for valid surrounding */
					if ((xx >= 0) && (yy >= 0) && (xx < size_x) && (yy < size_y)) {
						if (model[xx][yy] == WALL_ALL) {
							/* break wall and exit */
							removeWall(i,j,k);
							*ret_x = xx;
							*ret_y = yy;
							return 1;
						}
					}
				}

			}
		}
	}
	return 0;
}

static void removeWall(int x, int y, int dir) {
	int xx = x + offset_x[dir];
	int yy = y + offset_y[dir];
	model[x][y] -= 1 << dir;
	if ((xx >= 0) && (yy >= 0) && (xx < size_x) && (yy < size_y)) {
		model[xx][yy] -= 1 << ((dir+2)%4);
	}
}

int havewall(int currentX,int currentY,int dir) {
	return (model[currentX][currentY] & (1 << dir));
}

static int findRandomFreeDir(int x, int y) {
	int i;
	int j;
	int res[4] = {0,0,0,0};
	int sum = 0;
	int rnd;

	sum = getSuroundingFreeCell(x,y,res);
	if (sum != 0) {
		rnd = myrandom(sum);

		for (i=0,j=0; i<4; i++) {
			if (res[i]) {
				if (j == rnd) {
					return i;
				}
				j++;
			}
		}
	}
	return -1;
}

static int getSuroundingFreeCell(int x, int y, int res[4]) {
	int i;
	int sum = 0;
	int xx;
	int yy;

	for (i=0; i<4; i++) {
		xx = x + offset_x[i];
		yy = y + offset_y[i];
		if ((xx >= 0) && (yy >= 0) && (xx < size_x) && (yy < size_y)) {
			if (model[xx][yy] == WALL_ALL) {
				res[i] = 1;
				sum++;
			}
		}
	}
	return sum;
}

void cleanup(void) {
	if (model != NULL) {
		int i = 0;
		for (i=0; i<size_x; i++) {
			free(model[i]);
		}
		free(model);
	}
}
