/*
 *  Licensed Materials - Property of IBM / HCL Technologies
 *  Restricted Material of IBM / HCL Technologies
 *  IBM Rational Test RealTime / HCL OneTest
 *  (c) Copyright IBM Corp. 2020 All Rights Reserved.
 *
 *  US Governement Users Restricted Rights - Use, duplication or
 *  disclosure restricted by GSA ADP Schedule Contract with IBM Corp.
 *
 *       |\      _,,,---,,_
 *       /,`.-'`'    -.  ;-;;,_
 *      |,4-  ) )-,_..;\ (  `'-'
 *     '---''(_/--'  `-'\_)
 *
 ********************************************
 */
/*
 * main entry point
 * this project create a mini labyrinth and let you play inside
 * all display is done in ascii art, all keyboard interaction need a "enter" after option is selected
 * options on launch :
 * 	- help : display available parameters
 * 	-x : horizontal size (default = 30)
 * 	-y : vertical size (default = 25)
 * 	-seed : random seed, to have replayable tests
 * 	-noplay : just display maze, no interractive play
 */
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>
#include "core.h"
#include "game.h"
#include "display.h"

static int analyzeArgs( int argc, const char* argv[],int *noplay);

int main( int argc, const char* argv[] ) {
	int noplay = 0;
	if (analyzeArgs(argc,argv,&noplay) == 1) {
		initialize();
		if (noplay) {
			displayMatrix();
		} else {
			play();
		}
		cleanup();
	}
	return 0;
}

static int analyzeArgs( int argc, const char* argv[], int *noplay) {
	int overloadedSeed = 0;
	unsigned int randomSeed = 0;
	int cpt;
	int val;
	const char* arg;

	size_x = 8;
	size_y = 8;

	for (cpt = 1; cpt < argc; cpt++) {
		arg = argv[cpt];
		if (strcmp("-help",arg) == 0) {
			displayHelp();
			return 0;
		} else {
			if (sscanf(arg,"-seed=%d",&randomSeed) == 1) {
				overloadedSeed = 1;
			} else {
				if (sscanf(arg,"-x=%d",&val) == 1) {
					size_x = val;
				} else {
					if (sscanf(arg,"-y=%d",&val) == 1) {
						size_y = val;
					} else {
						if (strcmp("-noplay",arg) == 0) {
							*noplay = 1;
						} else {
							/* unknown option */
							displayError();
							displayHelp();
							return 0;
						}
					}
				}
			}
		}

	}
	if (overloadedSeed != 1) {
		time_t t;

		randomSeed = (unsigned) time(&t);
	}
	printf("seed : %d\n",randomSeed);
	printf("size_x : %d\n",size_x);
	printf("size_y : %d\n",size_y);
	srand(randomSeed);
	return 1;
}



